@@ -7,7 +7,7 @@ except:
 	pass # it's ok, it's not crucial
 __all__ = ["example_ylm"]
 
-def xyz(shape=128, limits=[-3, 3], spherical=False, sparse=True):
+def xyz(shape=128, limits=[-3, 3], spherical=False, sparse=True, centers=False):
 	dim = 3
 	try:
 		shape[0]
@@ -17,7 +17,11 @@ def xyz(shape=128, limits=[-3, 3], spherical=False, sparse=True):
 		limits[0][0]
 	except:
 		limits = [limits] * dim
-	v = [slice(vmin, vmax+(vmax-vmin)/float(N)/2, (vmax-vmin)/float(N-1)) for (vmin, vmax), N in zip(limits, shape)]
+	if centers:
+		print([(   vmin+(vmax-vmin)/float(N)/2, vmax-(vmax-vmin)/float(N)/4, (vmax-vmin)/float(N)) for (vmin, vmax), N in zip(limits, shape)])
+		v = [slice(vmin+(vmax-vmin)/float(N)/2, vmax-(vmax-vmin)/float(N)/4, (vmax-vmin)/float(N)) for (vmin, vmax), N in zip(limits, shape)]
+	else:
+		v = [slice(vmin, vmax+(vmax-vmin)/float(N)/2, (vmax-vmin)/float(N-1)) for (vmin, vmax), N in zip(limits, shape)]
 	if sparse:
 		x, y, z = np.ogrid.__getitem__(v)
 	else:
@@ -37,7 +41,7 @@ def example_ylm(m=0, n=2, shape=128, limits=[-4, 4], **kwargs):
 	data = np.abs(scipy.special.sph_harm(m, n, theta, phi) ** 2) * radial
 	return ipyvolume.volshow(data=data.T, **kwargs)
 
-def ball(rmax=3, rmin=0, shape=128, limits=[-4, 4], **kwargs):
+def ball(rmax=3, rmin=0, shape=128, limits=[-4, 4], display=True, **kwargs):
 	__, __, __, r, theta, phi = xyz(shape=shape, limits=limits, spherical=True)
 	data = r * 0
 	data[(r < rmax) & (r >= rmin)] = 0.5
@@ -45,6 +49,9 @@ def ball(rmax=3, rmin=0, shape=128, limits=[-4, 4], **kwargs):
 		kwargs["data_min"] = 0
 	if "data_max" not in kwargs:
 		kwargs["data_max"] = 1
-	return ipyvolume.volshow(data=data.T, **kwargs)
+	if display:
+		return ipyvolume.quickvolshow(data=data.T, **kwargs)
+	else:
+		return data.T
 
 # http://graphics.stanford.edu/data/voldata/
