@@ -1,14 +1,16 @@
 import ipywidgets as widgets
 import ipywidgets
 
-from traitlets import Unicode
+from traitlets import Unicode, Integer
 import traitlets
 from traittypes import Array
 import logging
 import numpy as np
-from .serialize import array_cube_png_serialization, array_serialization
+from .serialize import array_cube_png_serialization, array_serialization, array_binary_serialization, \
+    create_array_binary_serialization
 from .transferfunction import *
 import warnings
+import ipyvolume
 
 logger = logging.getLogger("ipyvolume")
 
@@ -20,16 +22,21 @@ class Scatter(widgets.DOMWidget):
     _view_module = Unicode('ipyvolume').tag(sync=True)
     _model_name = Unicode('ScatterModel').tag(sync=True)
     _model_module = Unicode('ipyvolume').tag(sync=True)
-    x = Array(default_value=None).tag(sync=True, **array_serialization)
-    y = Array(default_value=None).tag(sync=True, **array_serialization)
-    z = Array(default_value=None).tag(sync=True, **array_serialization)
-    vx = Array(default_value=None,allow_none=True).tag(sync=True, **array_serialization)
-    vy = Array(default_value=None,allow_none=True).tag(sync=True, **array_serialization)
-    vz = Array(default_value=None,allow_none=True).tag(sync=True, **array_serialization)
+    x = Array(default_value=None).tag(sync=True, **create_array_binary_serialization('x'))
+    y = Array(default_value=None).tag(sync=True, **create_array_binary_serialization('y'))
+    z = Array(default_value=None).tag(sync=True, **create_array_binary_serialization('z'))
+    vx = Array(default_value=None,allow_none=True).tag(sync=True, **create_array_binary_serialization('vx'))
+    vy = Array(default_value=None,allow_none=True).tag(sync=True, **create_array_binary_serialization('vy'))
+    vz = Array(default_value=None,allow_none=True).tag(sync=True, **create_array_binary_serialization('vz'))
     selected = Array(default_value=None,allow_none=True).tag(sync=True, **array_serialization)
-    size = traitlets.Float(0.01).tag(sync=True)
+    sequence_index = Integer(default_value=0).tag(sync=True)
+    size = traitlets.Union([traitlets.Float().tag(sync=True),
+                           Array(default_value=None,allow_none=True).tag(sync=True, **array_serialization)],
+                           default_value=0.1).tag(sync=True)
     size_selected = traitlets.Float(0.02).tag(sync=True)
-    color = traitlets.Unicode(default_value="red").tag(sync=True)
+    color = traitlets.Union([Unicode().tag(sync=True),
+                             Array(default_value=None,allow_none=True).tag(sync=True, **array_serialization)],
+                             default_value="red").tag(sync=True)
     color_selected = traitlets.Unicode(default_value="white").tag(sync=True)
     geo = traitlets.Unicode('diamond').tag(sync=True)
 
@@ -43,7 +50,7 @@ class VolumeRendererThree(widgets.DOMWidget):
     _model_name = Unicode('VolumeRendererThreeModel').tag(sync=True)
     _model_module = Unicode('ipyvolume').tag(sync=True)
 
-    data = Array(default_value=None, allow_none=True).tag(sync=True, **array_cube_png_serialization)
+    volume_data = Array(default_value=None, allow_none=True).tag(sync=True, **array_cube_png_serialization)
     data_min = traitlets.CFloat().tag(sync=True)
     data_max = traitlets.CFloat().tag(sync=True)
     tf = traitlets.Instance(TransferFunction, allow_none=True).tag(sync=True, **ipywidgets.widget_serialization)
@@ -53,14 +60,20 @@ class VolumeRendererThree(widgets.DOMWidget):
     scatters = traitlets.List(traitlets.Instance(Scatter), [], allow_none=False).tag(sync=True, **ipywidgets.widget_serialization)
 
     animation = traitlets.Float(1000.0).tag(sync=True)
+    animation_exponent = traitlets.Float(.5).tag(sync=True)
 
     ambient_coefficient = traitlets.Float(0.5).tag(sync=True)
     diffuse_coefficient = traitlets.Float(0.8).tag(sync=True)
     specular_coefficient = traitlets.Float(0.5).tag(sync=True)
     specular_exponent = traitlets.Float(5).tag(sync=True)
     stereo = traitlets.Bool(False).tag(sync=True)
+    screen_capture_enabled = traitlets.Bool(False).tag(sync=True)
+    screen_capture_mime_type = traitlets.Unicode(default_value='image/png').tag(sync=True)
+    screen_capture_data = traitlets.Unicode(default_value=None, allow_none=True).tag(sync=True)
     fullscreen = traitlets.Bool(False).tag(sync=True)
 
+    camera_control = traitlets.Unicode(default_value='trackball').tag(sync=True)
+
     width = traitlets.CInt(500).tag(sync=True)
     height = traitlets.CInt(400).tag(sync=True)
     downscale = traitlets.CInt(1).tag(sync=True)
@@ -69,22 +82,17 @@ class VolumeRendererThree(widgets.DOMWidget):
     xlim = traitlets.List(traitlets.CFloat, default_value=[0, 1], minlen=2, maxlen=2).tag(sync=True)
     ylim = traitlets.List(traitlets.CFloat, default_value=[0, 1], minlen=2, maxlen=2).tag(sync=True)
     zlim = traitlets.List(traitlets.CFloat, default_value=[0, 1], minlen=2, maxlen=2).tag(sync=True)
+
+    xlabel = traitlets.Unicode("x").tag(sync=True)
+    ylabel = traitlets.Unicode("y").tag(sync=True)
+    zlabel = traitlets.Unicode("z").tag(sync=True)
+
+    style = traitlets.Dict(default_value=ipyvolume.style.default).tag(sync=True)
+
     #xlim = traitlets.Tuple(traitlets.CFloat(0), traitlets.CFloat(1)).tag(sync=True)
     #y#lim = traitlets.Tuple(traitlets.CFloat(0), traitlets.CFloat(1)).tag(sync=True)
     #zlim = traitlets.Tuple(traitlets.CFloat(0), traitlets.CFloat(1)).tag(sync=True)
 
-    def _ipython_display_(self, **kwargs):
-        global _last_volume_renderer
-        try:
-            super(VolumeRendererThree, self)._ipython_display_(**kwargs)
-            _last_volume_renderer = None
-            print("reset view")
-        except:
-            raise
-
-
-
-
 def _volume_widets(v, lighting=False):
     import ipywidgets
     #angle1 = ipywidgets.FloatSlider(min=0, max=np.pi*2, value=v.angle1, description="angle1")
@@ -168,10 +176,14 @@ def quickvolshow(data, lighting=False, data_min=None, data_max=None, tf=None, st
     :return:
 
     """
-    if tf is None:
+    if tf is None: # TODO: should this just call the pylab interface?
         #tf = TransferFunctionJsBumps(**kwargs)
         tf_kwargs = {}
-        # opacity and widths can be scalars
+        # level, opacity and widths can be scalars
+        try:
+            level[0]
+        except:
+            level = [level]
         try:
             opacity[0]
         except:
@@ -201,7 +213,7 @@ def quickvolshow(data, lighting=False, data_min=None, data_max=None, tf=None, st
         data_min = np.nanmin(data)
     if data_max is None:
         data_max = np.nanmax(data)
-    v = VolumeRendererThree(data=data, data_min=data_min, data_max=data_max, stereo=stereo,
+    v = VolumeRendererThree(volume_data=data, data_min=data_min, data_max=data_max, stereo=stereo,
                             width=width, height=height,
                             ambient_coefficient=ambient_coefficient,
                             diffuse_coefficient=diffuse_coefficient,
@@ -220,4 +232,3 @@ def scatter(x, y, z, color=(1,0,0), s=0.01):
     fig.scatter = Scatter(x=x, y=y, z=z, color=color, size=s)
     fig.volume.scatter = fig.scatter
     return fig
-
